-- Luser global data
-- (c) zoner

local D = {         -- global object

    t      = 0,     -- current timestamp, millis
    reset  = true,  -- reset all
    exited = false, -- stop server and exit
    restart= false, -- restart all

    parg = require('parg'),              -- parsed arguments
    ini = require('kvini'):new():open('luser.ini'), -- application settings
    kv = {},                             -- key-value storage
    db = require('kvdb'):new():open('luser.db'),    -- key-value DB
    serv = require('serv'),

    channels={},   -- channels data
    values={},     -- current values
    tasks={},      -- tasks

    ports={        -- used/free ports
        used={},
        free={},
    },

    yield = coroutine.yield,
    wrap = coroutine.wrap,

    -- get new timestamp
    clock = function(self) self.t = os.clock() * 1000; return self.t end,
    -- timestamp after delay, ms
    ms = function(self,ms) self.t = self.D.t + ms end,
    -- timestamp after delay, s
    s  = function(self,s)  self.t = self.D.t + s * 1000 end,

    -- add new task f with data t
    addTask = function(self,f,t)
        t     = t or {}               -- task data
        t.run = self.wrap(f)          -- task coroutine
        t.ms  = self.ms; t.s = self.s -- milliseconds/seconds delay
        t.d   = t; t.D = self
        t.port = false                -- opened port
        t.yield = coroutine.yield,
        table.insert(self.tasks,t)
    end,

    -- close all
    close = function(self)
        self.ini:close()
        self.db.close()
    end,
    -- exit
    exit = function(self)
        self:close()
        os.exit()
    end,
}

return D
