-- Serial communication channel
-- (c) zöner

local lfs,rs232 = require('lfs'),require('luars232')
local Port = require('port')
local PortSerial = Port:new()

PortSerial.rs232 = rs232
PortSerial.bauds = {
    [300] = rs232.RS232_BAUD_300,
    [2400] = rs232.RS232_BAUD_2400,
    [4800] = rs232.RS232_BAUD_4800,
    [9600] = rs232.RS232_BAUD_9600,
    [19200] = rs232.RS232_BAUD_19200,
    [38400] = rs232.RS232_BAUD_38400,
    [57600] = rs232.RS232_BAUD_57600,
    [115200] = rs232.RS232_BAUD_115200,
    [460800] = rs232.RS232_BAUD_460800
}
PortSerial.bits = {
    [5] = rs232.RS232_DATA_5,
    [6] = rs232.RS232_DATA_6,
    [7] = rs232.RS232_DATA_7,
    [8] = rs232.RS232_DATA_8
}
PortSerial.stops = {
    [1] = rs232.RS232_STOP_1,
    [2] = rs232.RS232_STOP_2
}
PortSerial.parity = {
    ['none'] = rs232.RS232_PARITY_NONE,
    ['odd'] = rs232.RS232_PARITY_ODD,
    ['even'] = rs232.RS232_PARITY_EVEN
}

function PortSerial:names()
    local ports = {}
    local pnames = {}
    if os.getenv('COMSPEC') == nil then
        for fn in lfs.dir('/dev') do
            if (fn:len() > 3) and (string.find(fn,'tty') == 1) then
                table.insert(pnames, '/dev/' .. fn)
            end
        end
    else for i=1,64 do table.insert(pnames, 'COM' .. i_) end
    end
    for _,pn in pairs(pnames) do
        local f,_ = io.open(pn, 'rb')
        if f ~= nil then table.insert(ports, pn); f:close() end
    end
    table.sort(ports)
    return ports
end

function PortSerial:open(name)
    self.name = name
    self:close()
    local e,p = rs232.open(name)
    if e == rs232.RS232_ERR_NOERROR then
        self.port = p
        p:set_flow_control(rs232.RS232_FLOW_OFF)
    end
end
function PortSerial:opened() return self.port ~= nil end
function PortSerial:close()
    if self.port ~= null then
        self.port:close()
        self.port = nil
    end
end

function PortSerial:read() if self:opened() then return self.port:read() else return '' end end
function PortSerial:write(s) if self:opened() then self.port:write(s) end return self end

function PortSerial:props() return {'name','baud','bits','parity','stops','dtr','dsr','dcd','rts','cts','ri'} end
function PortSerial:prop(k,v)
    if self.port ~= nil then
        if v ~= nil then
            if (k == 'baud') and (self.bauds[tonumber(v)] ~= nil) then self.port:set_baud_rate(self.bauds[tonumber(v)])
            elseif (k == 'bits') and (self.bits[tonumber(v)] ~= nil) then self.port:set_data_bits(self.bits[tonumber(v)])
            elseif (k == 'stops') and (self.stops[tonumber(v)] ~= nil) then self.port:set_stop_bits(self.stops[tonumber(v)])
            elseif (k == 'parity') and (self.parity[v] ~= nil) then self.port:set_parity(self.parity[v])
            elseif (k == 'dtr') and (tonumber(v) <= 1) then self.port:set_dtr(tonumber(v))
            elseif (k == 'rts') and (tonumber(v) <= 1) then self.port:set_rts(tonumber(v))
            else
            end
            v = self
        else
            if k == 'name' then v = self.name
            elseif k == 'baud' then
                local b = self.port:baud_rate()
                for bn,bc in pairs(self.bauds) do
                    if bc == b then v = bn end
                end
            elseif k == 'bits' then
                local b = self.port:data_bits()
                for bn,bc in pairs(self.bits) do
                    if bc == b then v = bn end
                end
            elseif k == 'parity' then
                local b = self.port:parity()
                for bn,bc in pairs(self.parity) do
                    if bc == b then v = bn end
                end
            elseif k == 'stops' then
                local b = self.port:stop_bits()
                for bn,bc in pairs(self.stops) do
                    if bc == b then v = bn end
                end
            elseif k == 'dtr' then v = self.port:dtr()
            elseif k == 'rts' then v = self.port:rts()
            else
            end
        end
    else
        if v ~= nil then v = self end
    end
    return v
end

return PortSerial
